OK = 'OK'
BAD_OTP = 'BAD_OTP'
REPLAYED_OTP = 'REPLAYED_OTP'
BAD_SIGNATURE = 'BAD_SIGNATURE'
MISSING_PARAMETER = 'MISSING_PARAMETER'
NO_SUCH_CLIENT = 'NO_SUCH_CLIENT'
OPERATION_NOT_ALLOWED = 'OPERATION_NOT_ALLOWED'
BACKEND_ERROR = 'BACKEND_ERROR'
NOT_ENOUGH_ANSWERS = 'NOT_ENOUGH_ANSWERS'
REPLAYED_REQUEST = 'REPLAYED_REQUEST'
